;***	DDARRAY.LSP
;***	Dialog Box for Array		(c)1995,  Ken Leseberg
;***	Modified by Don J. Buschert	(c)1997
;
;	Email:	don.buschert@sait.ab.ca
;		buschert@spots.ab.ca
;	AutoCAD Page:	http://www.spots.ab.ca/~buschert/autocad/main.htm 
;
;	Disclaimer:
;	Permission to use, copy, modify, and distribute this software 
;	for any purpose and without fee is hereby granted, provided
;	that the above copyright notice appears in all copies and 
;	that both that copyright notice and the limited warranty and 
;	restricted rights notice below appear in all supporting 
;	documentation.
;
;	THIS PROGRAM IS PROVIDED "AS IS" AND WITH ALL FAULTS.  THE AUTHOR
;	SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF MERCHANTABILITY OR
;	FITNESS FOR A PARTICULAR USE.  THE AUTHOR ALSO DOES NOT WARRANT THAT
;	THE OPERATION OF THE PROGRAM WILL BE UNINTERRUPTED OR ERROR FREE.
;
;	Version 1.2 07/17/95
;	Added control measures to ensure all information regarding the
;	array is gathered for processing.
;
;	Version 1.1 05/30/95
;	Fixed the defaults function so the 	defaults are global
;
;***	Function ARRAY_CHECK
;Called after hitting OK, checks to see if all the data required to
;perform the array is present...
;
(defun ARRAY_CHECK ( / )
  (if (not aray_sst1);first check to see if objects were selected
    (alert "No objects selected for array")
    (if (= aray_type 0);if array type is rectangular...
      (progn
        ;if objects selected, check to see if rows are > 0
        (if (= aray_rows "0")
          (alert "Number of Rows must be at least 1...")
          ;check to see if columns are > 0
          (if (= aray_cols "0")
            (alert "Number of Columns must be at least 1...")
            ;check to see if rows and cols are both "1"
            (if (and (= aray_rows "1")(= aray_cols "1")) 
              (alert "Both Rows and Columns cannot be set to 1.\nThis would result in a one element array..." )
              ;if row is greater than 1 but row dist is 0
              (if (and (> (atof aray_rows) 1) (= (atof aray_rdis) 0))
                (alert "You must enter a Row Distance...")
                ;if col is greater than 1 but col dist is 0
                (if (and (> (atof aray_cols) 1) (= (atof aray_cdis) 0))
                  (alert "You must enter a Column Distance...")
                  (progn ;if all settings are correct, then perform array
                    (done_dialog);close dialog
                    (setq aray_flag 1);set flag
                  )
                )
              )
            )
          )
        )         
      )
      ;if array type is polar...
      (progn
        (if (<= (atoi aray_nump) 0);check to see number of items is more than 0
          (alert "Number of Items must be greater than 0...")
          (if (= (atof aray_fdeg) 0);if degrees equal to zero
            (alert "Degrees to Fill cannot be equal to 0...")
            (progn ;if all settings are correct, then perform array
              (done_dialog);close dialog
              (setq aray_flag 1);set flag             
            )
          )
        )
      )
    )

  ) 
)
;
;***	Function:  ARRAY_DEFAULTS
;Sets up the defaults for the dialog...
;
(defun ARRAY_DEFAULTS ()             

  (set_tile "x_pt" aray_poix)        
  (set_tile "y_pt" aray_poiy)
  (set_tile "z_pt" aray_poiz)

  (set_tile "how_many"
     (if (/= aray_sst1 nil)
        (rtos (sslength aray_sst1) 2 0)
     "0" )
  )              

  (setq aray_temn (atoi aray_nump))
  (setq aray_nump (rtos aray_temn 2 0))
  (set_tile "n_items" aray_nump)

  (set_tile "fill_deg" aray_fdeg)

  (if (= 0 aray_roti)
     (set_tile "rot_item" "0")
     (progn
        (set_tile "rot_item" "1")
        (setq aray_roti 1)
     )
  )

  (if (= 0 aray_type)
     (set_tile "rect" "1")
     ()
  )

  (if (= 1 aray_type)
     (set_tile "polr" "1")
     ()
  )

  (setq aray_temr (atoi aray_rows))
  (setq aray_rows (rtos aray_temr 2 0)) 
  (set_tile "rows" aray_rows)

  (setq aray_temc (atoi aray_cols))
  (setq aray_cols (rtos aray_temc 2 0))
  (set_tile "cols" aray_cols)

  (set_tile "r_dist" aray_rdis)
  (set_tile "c_dist" aray_cdis)
)                                  
;End of ARRAY_DEFAULTS function.
;
;***	Function:  ARRAY_DIALOG
;Runs the dialog..
(defun ARRAY_DIALOG ()

  (if (< (setq dcl_id (load_dialog "ddarray.dcl")) 0) (exit))

  (if (not aray_poix)(setq aray_poix "0.0"))		;initialize various items for the
  (if (not aray_poiy)(setq aray_poiy "0.0"))		;first time that the dialog box is used.
  (if (not aray_poiz)(setq aray_poiz "0.0"))		;x, y, & z all = 0.
  (if (not aray_nump)(setq aray_nump "4"))	;polar array for 4 items.
  (if (not aray_fdeg) 				;polar array for fill of 360 degress.
    (setq aray_fdeg "360.00")
  )
  (if (not aray_roti)(setq aray_roti 1))	;rotate items during polar array.
  (if (not aray_rows)(setq aray_rows "2"))		;2 rows for rectangular array.
  (if (not aray_cols)(setq aray_cols "2"))		;2 columns for rectangular array.
  (if (not aray_rdis)(setq aray_rdis "0.0"))	;row distance = 0
  (if (not aray_cdis)(setq aray_cdis "0.0"))	;column distance = 0
  (if (not aray_type)(setq aray_type 0))	;start dialog as rectangular array.
  (setq aray_sst1 nil)   
  (setq what_next 5)         

  (while (< 2 what_next)
     (if (not (new_dialog "ddarray" dcl_id)) (exit))

     (ARRAY_DEFAULTS)

     (if (= 1 aray_type);  If statement used to enable / disable
        (ARRAY_POLAR)     ;  tiles the first time through.  Also 
        (ARRAY_RECTANG)     ;  runs thru here each time thru loop.
     )                 ;  End of "if" statement.

     (action_tile "pick_pt"    "(done_dialog 4)")
     (action_tile "sel_objs"   "(done_dialog 3)")
     (action_tile "pick_xy"    "(done_dialog 6)")
     (action_tile "x_pt"       "(setq aray_poix $value)")         
     (action_tile "y_pt"       "(setq aray_poiy $value)")       
     (action_tile "z_pt"       "(setq aray_poiz $value)")
     (action_tile "n_items"    "(setq aray_nump $value)")       
     (action_tile "fill_deg"   "(setq aray_fdeg $value)")   
     (action_tile "rot_item"   "(setq aray_roti (atoi $value))")
     (action_tile "rows"       "(setq aray_rows $value)")
     (action_tile "cols"       "(setq aray_cols $value)")
     (action_tile "r_dist"     "(setq aray_rdis $value)")
     (action_tile "c_dist"     "(setq aray_cdis $value)")
     (action_tile "polr"       "(ARRAY_POLAR)")
     (action_tile "rect"       "(ARRAY_RECTANG)")
     
     (action_tile "accept" "(array_check)")
;    (action_tile "help" "(saithelp \"array\")");disabled for non-SAIT users

     (setq what_next (start_dialog)); Throw up the dialogue.

     (cond ; Decide what to do next.
        ;; If select objects was picked...
        ((= what_next 3)
           (setq aray_sst1 (ssget))
        )

        ;; If pick to show x,y row and column distances...
        ((= what_next 6)
           (initget 1)
           (setq aray_poi1 (getpoint "Pick first point: "))
           (setq aray_poi2 (getcorner aray_poi1 "Pick second point: "))
           (setq aray_rdis (rtos (- (cadr aray_poi2) (cadr aray_poi1)) 2 4))
           (setq aray_cdis (rtos (- (car aray_poi2) (car aray_poi1)) 2 4))
        )

        ;; If the base point was picked...
        ((= what_next 4)
           (initget 1)
           (setq aray_ppoi (getpoint "Polar array base point: "))
           (setq aray_poix (rtos (car aray_ppoi) 2 4))
           (setq aray_poiy (rtos (cadr aray_ppoi) 2 4))
           (setq aray_poiz (rtos (caddr aray_ppoi) 2 4))
        )
     ); end of decide "cond" 
  ); end of while loop. 

  (start_dialog)
  (unload_dialog dcl_id)
)
;end of ARRAY_DIALOG function.
 
;
;***	Function:  ARRAY_PERFORM
;This function performs the array based on the dialog settings.
(defun ARRAY_PERFORM ( / )
  (if (= 1 aray_type);if polar array...
    (progn
      (if (= 1 aray_roti)
        (setq aray_yorn "y")
        (setq aray_yorn "n")
      )
      ;set polar base point
      (setq aray_ppoi
        (list (atof aray_poix) (atof aray_poiy) (atof aray_poiz))
      )
      ;perform polar array...
      (command ".array" "prev" "" "p" aray_ppoi aray_nump aray_fdeg aray_yorn)
    )
    ;if rectangular array...
    (progn
      (if (= "1" aray_rows);if arraying columns only...
        (command ".array" "prev" "" "r" aray_rows aray_cols aray_cdis)
      )
      (if (= "1" aray_cols);if arraying rows only...
        (command ".array" "prev" "" "r" aray_rows aray_cols aray_rdis)
      )
      (if (and ;if arraying both rows and columns...
            (/= "1" aray_rows)
            (/= "1" aray_cols)
          )
        (command ".array" "prev" "" "r" aray_rows aray_cols aray_rdis aray_cdis)
      )
    )
  ); end of aray_type if statement.
)
 
;***	Function:  ARRAY_POLAR
;Sets tiles for Polar Array...
(defun ARRAY_POLAR ()
   (setq aray_type 1)
   (mode_tile "pick_pt" 0)
   (mode_tile "pick_xy" 1)   
   (mode_tile "x_pt" 0)   
   (mode_tile "y_pt" 0)
   (mode_tile "z_pt" 0)
   (mode_tile "n_items" 0)
   (mode_tile "fill_deg" 0)
   (mode_tile "rot_item" 0)
   (mode_tile "rows" 1)
   (mode_tile "cols" 1)
   (mode_tile "r_dist" 1)
   (mode_tile "c_dist" 1)
)
;End of the ARRAY_POLAR function.

;***	Function:  ARRAY_RECTANG
;Sets tiles for Rectangular Array...
(defun ARRAY_RECTANG ()
   (setq aray_type 0)
   (mode_tile "pick_pt" 1)
   (mode_tile "pick_xy" 0)   
   (mode_tile "x_pt" 1)   
   (mode_tile "y_pt" 1)
   (mode_tile "z_pt" 1)
   (mode_tile "n_items" 1)
   (mode_tile "fill_deg" 1)
   (mode_tile "rot_item" 1)
   (mode_tile "rows" 0)
   (mode_tile "cols" 0)
   (mode_tile "r_dist" 0)
   (mode_tile "c_dist" 0)
)
;End of the ARRAY_RECTANG function.


;***	Function C:DDARRAY
;Controls and executes program
(defun C:DDARRAY (/ 
    ;aray_cdis		;column distance
    ;aray_cols		;columns
    ;aray_fdeg		;fill degrees
    aray_flag		;control flag
    ;aray_nump		;number of items for polar array
    aray_poi1		;first point
    aray_poi2		;second point
    ;aray_poix		;x point for polar array
    ;aray_poiy		;y point for polar array
    ;aray_poiz		;z point for polar array
    aray_ppoi		;pick point
    ;aray_rdis		;row distance
    ;aray_roti		;rotate items
    ;aray_rows		;rows
    aray_sst1 		;Selection set
    aray_temc		;temporary column flag
    aray_temn		;temporary number flag
    aray_temr		;temporary row flag
    ;aray_type		;polar or rectangle type of array
    aray_yorn		;yes/no flag
    sv_cmdecho		;cmdecho
    what_next		;control for dialog
     )

   (setq sv_cmdecho (getvar "cmdecho"))
   (setvar "cmdecho" 0)
   
   (ARRAY_DIALOG)   
   (ARRAY_DEFAULTS); After exiting dialog, run the default function here
             ; to force input for rows, cols, n_items, etc. to integer.
      
   (if aray_flag
     (ARRAY_PERFORM);perform array 
   )
   (setvar "cmdecho" sv_cmdecho)
   (princ)
)
;; End of the main function.
